
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h> 

#include "Gkch2.h"
#include "uart.h" 
#include "spi.h"

#define BIT_RAZ   1
#define PORT_RAZ  PORTC
#define DDRT_RAZ  DDRC



#define ADR_TYPE 7
#define ADR_Fzad 8

#define T_AD9850 50
#define T_AD9851 51

#define BU_AD9850 0x00
#define BU_AD9851 0x01

#define MS1  250
#define MS01 MS1/10

#define Set_time(x) do{OCR1AH=(x >> 8); OCR1AL=(x & 0xFF);}while(0)

//  
#define COUNT_MF   (((END_FUN-1) >> 3)+1) /*  */
volatile uint8_t Flag_Prog[COUNT_MF];//    
volatile FUN Fun[END_FUN];           //  

struct F_ZAD  
{
 union 
 {
  uint32_t f;
  uint8_t  byte[4];
 };    
};

volatile struct F_ZAD f_zad; //  

double Kf;//   4294967296/F 

volatile uint8_t type_AD;// AD
volatile uint8_t b_cofig; //5 , 

struct Un_AD  
{
 union 
 {
  uint32_t word_fr;
  uint8_t  byte_fr[5];//5     DDS
 };     //  
};

volatile struct Un_AD
freq0,    //  
freq1,    //  
st,       //
freq_tek, //  
freq_spi; //    AD (freq_tek * K)

volatile uint8_t begin_gkch=1;//  
volatile uint8_t nom_acd=0;   //  
volatile uint8_t f_izm=0;     //      
volatile uint8_t pauz=2;      // 2ms

//------------------------------------------------------------------
static uint16_t ADC_res(unsigned char adc_input);
static void send_OK( uint8_t command);
//-----------------------------------------------------------------------------
ISR( TIMER1_COMPA_vect )
{
 f_izm=1;
}
//-----------------------------------------------------------------------------
void null_Prog (void)
{
}
//------------------------------------------------------------------------------
void set_AD9850(void)
{
 freq_spi.word_fr = freq_tek.word_fr * Kf + 0.5;//   AD ( )     	
 freq_spi.byte_fr[4] = b_cofig;  

 SPI_WriteArray(5,(void *)freq_spi.byte_fr);//  

 SPI_PORTX |= (1<<SPI_SS);// 
 SPI_PORTX &= ~(1<<SPI_SS); 
}
//------------------------------------------------------------------------------------
void fr0(void)
{ uint8_t i;

 for(i=0;i<4;i++)freq0.byte_fr[i]=rx_buffer[i];
 freq0.byte_fr[4]=0;

 freq_tek.word_fr=freq0.word_fr;

 set_AD9850();//     AD9850 

 send_OK(Fr0);
}
//-----------------------------------------------------------------------------------------
void fr1(void)
{uint8_t i;
 for(i=0;i<4;i++)freq1.byte_fr[i]=rx_buffer[i];
 freq1.byte_fr[4]=0;
 send_OK(Fr1);
}
//-----------------------------------------------------------------------------------------
void step(void)
{uint8_t i;
 for(i=0;i<4;i++)st.byte_fr[i]=rx_buffer[i];
 st.byte_fr[4]=0;
 send_OK(Step);
}
//------------------------------------------------------------------------------------------
static void start_gkch(void)
{
 begin_gkch=255;
 send_OK(Start_Gkch);
}
//-------------------------------------------------------------------------------------------
static void stop_gkch(void)
{
 begin_gkch=0;
 send_OK(Stop_Gkch);
}
//------------------------------------------------------------------------------------------
static void v5(void)
{
 ADMUX &= ~(1<<REFS1);//  5 
 send_OK(V5);
}
//-----------------------------------------------------------------------
static void v1(void)
{
 ADMUX |= (1<<REFS1);//  1.1 
 send_OK(V1);
}
//---------------------------------------------------------------------------------
static void a0(void)
{
 nom_acd=0;
 send_OK(A0);
}
//----------------------------------------------------------------------------------
static void a2(void)
{ 
 nom_acd=2; 
 send_OK(A2);
}
//---------------------------------------------------------------------------------
static void a3(void)
{
 nom_acd=3;
 send_OK(A3);
}
//----------------------------------------------------------------------------------
static void a4(void)
{ 
 nom_acd=4; 
 send_OK(A4);
}
//---------------------------------------------------------------------------------

static void set_Fzad(void)//    EEPROM
{ 
 eeprom_update_dword((uint32_t *)ADR_Fzad,f_zad.f); 
 eeprom_update_byte((uint8_t *)ADR_TYPE,type_AD);
 send_OK(SetFzad);
}
//-----------------------------------------------------------------------------
static void f_zg(void)//   
{
 uint8_t i,crc;

 crc=':';crc+=8;crc+=Get_Fzg;

 uart_putc(':'); uart_putc(8);  uart_putc(Get_Fzg);

 for(i=0;i<4;i++)
   {
    crc+=f_zad.byte[i];
    uart_putc(f_zad.byte[i]);
   }
 // AD9850 = 50, AD9851 = 51
 crc+=type_AD;
 uart_putc(type_AD);

 uart_putc(crc);
}
//-------------------------------------------------------------------------------------------
static void izm_fk (void)//
{ 
 uint8_t i;

 for(i=0;i<4;i++)f_zad.byte[i]=rx_buffer[i];

 type_AD=rx_buffer[i];
 b_cofig=type_AD-50;
 
 Kf=(double)4294967296 / (double)f_zad.f ;

 send_OK(Izm_Fk);
 
 Flag_Prog[Get_Fzg >> 3] |= (1 << (Get_Fzg & 0x07));//      
}
//----------------------------------------------------------------------------------------------------
// ':' 1,   CRC 2,  3, CRC 4
static void send_OK( uint8_t command)
{
 uint8_t crc;
       crc=':';       crc+=3;        crc+=command;  
 uart_putc(':'); uart_putc(3);  uart_putc(command);  uart_putc(crc);
}

//       
//   
static uint16_t ADC_res(unsigned char adc_input)
{
  uint8_t i;
  uint16_t temp=0;

  ADMUX=adc_input | (ADMUX & 0xF0);

  for(i=0;i<8;i++)
   {
    ADCSRA |= (1<<ADSC);

    while ((ADCSRA & (1 << ADSC)) != 0);  //,     
	temp+=ADCW;
   }
   
  temp+=4;
  temp >>=3;
  return temp ; 
}

//-----------------------------------------------------------------------------------

 //__attribute__((optimize("O0")))   
//void __attribute__((optimize("O0"))) GkchExe (void)
void GkchExe (void)
{ 
  static struct Un_AD fr_old;          
  uint8_t crc; 

  uint16_t temp_ADC = ADC_res(nom_acd);//  
   
//// 
   DDRT_RAZ |=(1<<BIT_RAZ);//    0,     
   SPI_PORTX |=  (1<<SPI_SS);
   crc=':'; crc+=9; crc+=Out_ADC; //  SPI_SS
   SPI_PORTX &= ~(1<<SPI_SS); 
   DDRT_RAZ &=~(1<<BIT_RAZ); //  ( Z),    

//  
   //crc=':'; crc+=9; crc+=Out_ADC;  
   uart_putc(':'); uart_putc(9);  uart_putc(Out_ADC);

   crc+=fr_old.byte_fr[0]; crc+=fr_old.byte_fr[1]; crc+=fr_old.byte_fr[2]; crc+=fr_old.byte_fr[3];  
   uart_putc(fr_old.byte_fr[0]); uart_putc(fr_old.byte_fr[1]); uart_putc(fr_old.byte_fr[2]);uart_putc(fr_old.byte_fr[3]); 
   	  	  	  
// ADC  	 
   crc+=(temp_ADC & 0x00ff);   crc+=(temp_ADC >> 8);
   uart_putc((uint8_t)(temp_ADC & 0x00ff)); uart_putc((uint8_t)(temp_ADC >> 8));        
   uart_putc(crc); //    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   

  if(freq_tek.word_fr !=0 )//      
    {  
	 //10 
     unsigned int time_ms=100000 / freq_tek.word_fr;
	 Set_time((MS01 * time_ms + MS1) * pauz);  
    } 
     	  	 
  if(begin_gkch != 0)
    {
  // 
     freq_tek.word_fr+=st.word_fr;
     
     if(freq_tek.word_fr > freq1.word_fr)// 
	   {
		//Set_time(20 * MS1);// 
		PORTD |= 0x80;	
		
		freq_tek.word_fr=freq0.word_fr;

		_delay_us(500);

		PORTD &= ~0x80;
       }
    }

  fr_old.word_fr = freq_tek.word_fr; 
        
  freq_spi.word_fr = (float)freq_tek.word_fr * Kf + 0.5;//   AD ( )     	
  freq_spi.byte_fr[4] = b_cofig;  

  SPI_WriteArray(5,(void *)freq_spi.byte_fr);//  	 	   
}

//-----------------------------------------------------------------------------------------------------
void set_pauz(void)
{
 pauz=rx_buffer[0];
 send_OK(SET_PAUZ);
}
//-----------------------------------------------------------------------------------------------------------
uint32_t my_temp;

int main( void )
{ 
 uint8_t  i;
 uint8_t  sdvig;
 uint8_t  mf;
 
 for(i=0;i< COUNT_MF; i++)Flag_Prog[i]=0; 

 Fun[Fr0]=fr0;
 Fun[Fr1]=fr1;
 Fun[Step]=step;
 
 Fun[Start_Gkch]=start_gkch;
 Fun[Stop_Gkch]=stop_gkch;

 Fun[V5]=v5;
 Fun[V1]=v1;

 Fun[A0]=a0;
 Fun[A2]=a2;
 Fun[A3]=a3;
 Fun[A4]=a4;

 Fun[SetFzad]=set_Fzad;
 Fun[Izm_Fk]  = izm_fk;
 Fun[Get_Fzg] = f_zg;
 Fun[SET_PAUZ]=set_pauz;

 // Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 250,000 kHz
// Mode: CTC top=OCR1A
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: On
// Compare B Match Interrupt: Off
 TCCR1A=0x00;
 TCCR1B=0x0B;
 TCNT1H=0x00;
 TCNT1L=0x00;
 ICR1H=0x00;
 ICR1L=0x00;

 OCR1AH=0x00;
 OCR1AL=0xFA;
 OCR1BH=0x00;
 OCR1BL=0x00;

 TIMSK1=0x02;

 //ADC
 ADCSRA=0x87;//     128
 ADMUX=(1<<REFS0);//AVCC      ,0   
 DDRC &=~(1<<PC0 | 1<<PC2 | 1<<PC3 | 1<<PC4);//  0, 2, 3, 4
 PORTC &=~(1<<PC0 | 1<<PC2 | 1<<PC3 | 1<<PC4);// 
 
 PORT_RAZ &=~(1<<BIT_RAZ);//0
 DDRT_RAZ |=(1<<BIT_RAZ);//   C
 DDRT_RAZ &=~(1<<BIT_RAZ);// (Z)

 DDRD |= (1 << 7); //  
 
 init_uart();
 SPI_Init(); 

 freq_tek.word_fr=0;

 type_AD = eeprom_read_byte ((uint8_t *) ADR_TYPE); 

 f_zad.f = eeprom_read_dword ((uint32_t *) ADR_Fzad); 

 if(type_AD == T_AD9850)
  {b_cofig = BU_AD9850;}
 else if(type_AD == T_AD9851)
        {b_cofig = BU_AD9851; }
       else 
	    { 
		 type_AD = T_AD9850;
		 b_cofig = BU_AD9850;
		 f_zad.f = 125000000L;
		}
 
 Kf=(double)4294967296 / (double)f_zad.f ;

 asm("sei");

 //freq_tek.word_fr=1000000;//  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 while (1)
  { 
   for(i=0, mf=0; mf < COUNT_MF; mf++) 
    {
	 if(Flag_Prog[mf] == 0){i+=8; continue;}
     for(sdvig = 0x01; sdvig != 0 ;sdvig <<= 1,i++)
      {
       if(Flag_Prog[mf] & sdvig)
         {
	      Flag_Prog[mf] &=  ~sdvig;// 
	      Fun[i]();
	     }
      }
    }

   if(f_izm !=0 )//    
    {
	 f_izm = 0;
     GkchExe ();
	 //f_izm = 1;
    }
  }

  return 0;
}
















