

#define BAUD  57600
//#define BAUD  115200U
#include <avr/interrupt.h>

#include "Gkch2.h"
#include "uart.h"
#include <util/setbaud.h>

#define FRAMING_ERROR (1<<FE0)
#define PARITY_ERROR (1<<UPE0)
#define DATA_OVERRUN (1<<DOR0)

/*  
1  ':' 2  -  -2, 3  -  ,  ...  ,   CRC
*/

void init_uart(void)
{

 UCSR0A = (1<< U2X0);
 UCSR0B = ((1 << RXCIE0) | (1 << UDRIE0) | (1 << RXEN0) | (1 << TXEN0));

 UCSR0C = 0x06; //011 - 8 

 UBRR0H = UBRRH_VALUE;
 UBRR0L = UBRRL_VALUE;


/*
 UCSR0B = ((1 << RXCIE0) | (1 << UDRIE0) | (1 << RXEN0) | (1 << TXEN0));

 UBRR0H = UBRRH_VALUE;
 UBRR0L = UBRRL_VALUE;
#if USE_2X
 UCSR0A |= (1 << U2X0);
#else
 UCSR0A &= ~(1 << U2X0);
#endif
*/
}

#define FLAG_IN 
#define RX_BUFFER_SIZE 64

volatile uint8_t rx_buffer[RX_BUFFER_SIZE];
volatile uint8_t rx_counter=0;

ISR(USART_RX_vect)
{
 int8_t status,data;
 static uint8_t len;
 static uint8_t crc;
 static uint8_t kod_command=255;
 static uint8_t obr=0;

 data=UDR0; 
status=UCSR0A;
 
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
 {
  switch(obr)
   {
    case 0: if(data==':'){ obr=1; rx_counter=0;crc=':';}break;

    case 1:
      {
        len = data; 
        if(len > RX_BUFFER_SIZE-1 || len < 3) obr=0;           
        else obr=2;      
        crc+=len;
        len-=3;//   3  - 0=':', 1=len, 2=    
      } break;

    case 2:
      {
       kod_command=data;
       crc+=kod_command;
       obr=3;
      }break;

    default :
      {
       if(rx_counter >= len) // CRC
        {    
         if(crc==data)
           { 
			Flag_Prog[kod_command >> 3] |= (1 << (kod_command & 0x07));//    
           }

          obr=0;
          return; 
        }
       
       rx_buffer[rx_counter++]=data;
       crc+= data;
      }

   } //switch

 }
else obr=0; //    ':'
 
}

//=================================================================================
// USART Transmitter buffer

#define TX_BUFFER_SIZE 64
volatile uint8_t tx_buffer[TX_BUFFER_SIZE];

volatile uint8_t tx_wr_index, tx_rd_index, tx_counter=0;

//---------------------------------------------------------------------------


ISR( USART_UDRE_vect )
{
 if (tx_counter )
   {
      --tx_counter;
      UDR0=tx_buffer[tx_rd_index++];
      if (tx_rd_index == TX_BUFFER_SIZE) tx_rd_index=0;
   }
 else UCSR0B &= (~(1 << UDRIE0)); //  UDR   
}

//-------------------------------------------------------------
void uart_putc(  char c ) 
{
 while (tx_counter == TX_BUFFER_SIZE);

 if (tx_counter || ((UCSR0A & (1 << UDRE0))==0))
  {
   asm("cli");
   tx_buffer[tx_wr_index++]=c;
   if (tx_wr_index == TX_BUFFER_SIZE) tx_wr_index=0;   
   ++tx_counter;  
   asm("sei"); 
  }
 else
  {
   UDR0=c;
  }

  UCSR0B |= (1 << UDRIE0); //  UDR

}

